#!/usr/bin/python3

import sys

APP="mdvinput"
DIR="/usr/share/locale"

try:
	import gi
	gi.require_version("Gtk", "3.0")
except:
	print("You do not have the required PyGTK version.")
	sys.exit(1)
try:
	from gi.repository import Gtk
	import os
	import shutil
	import gettext
	import configparser
	import getpass
	import locale
except:
	print("You do not have the required dependencies")
	sys.exit(1)

# This is needed to make Gtk.Builder work by specifying the
# translations directory
locale.bindtextdomain(APP, DIR)
 
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)

try:
	lang = gettext.translation(APP, DIR)
	_ = lang.gettext
except:
	print("WARNING: No translations found for your language, using default")

gettext.install(APP, DIR)

class mdvinput:
	def __init__(self):
		
		self.glade = '/usr/share/mdvinput/mdvinput.glade'
		self.builder = Gtk.Builder()
		self.builder.set_translation_domain(APP)
		self.builder.add_from_file(self.glade)
		self.window = self.builder.get_object("dlg")
		self.window.set_title(_("Configure keyboard, mouse and other input devices"))
		self.window.set_title(_("Keyboard, Mouse and Touchpad"))
		self.window.connect("destroy", Gtk.main_quit)
		self.window.show()

		# read config file

		config = configparser.RawConfigParser()
		config.add_section('Mouse')
		config.add_section('Keyboard')
		config.add_section('Touchpad')
		cur_user = getpass.getuser()
		config_dir = '/home/' + cur_user + '/.config/lxsession/LXDE/'
		xdg_file_name = '/etc/xdg/lxsession/LXDE/desktop.conf'
		conf_file_name = config_dir + 'desktop.conf'
		
		# touch file. If not, copy from /etc/xdg/lxsession/LXDE
		
		try:
			if not os.path.isdir(config_dir):
				os.makedirs(config_dir)
			if ((not os.path.exists(conf_file_name)) and (os.path.exists(xdg_file_name))):
				# copy file template
				shutil.copy(xdg_file_name, conf_file_name)
		finally:
			config.read(conf_file_name)
		
		# keyboard delay
		delay = self.builder.get_object("kb_delay")
		delay.set_range(100, 1100)
		delay.set_increments(100, 100)
		delay.set_digits(0)
		if (config.has_option('Keyboard', 'Delay')):
			delay.set_value(config.getint('Keyboard', 'Delay'))

		# interval
		interval = self.builder.get_object("kb_interval")
		interval.set_range(1, 200)
		interval.set_increments(1, 1)
		interval.set_digits(0)
		if (config.has_option('Keyboard', 'Interval')):
			interval.set_value(config.getint('Keyboard', 'Interval'))

		# touchpad
		if (config.has_option('Touchpad', 'Disable')):
			self.builder.get_object("touchpad_disable").set_active(config.getboolean('Touchpad', 'Disable'))
		if (config.has_option('Touchpad', 'VertScroll')):
			self.builder.get_object("vertical_scroll").set_active(config.getboolean('Touchpad', 'VertScoll'))
		if (config.has_option('Touchpad', 'TypeOff')):
			self.builder.get_object("touchpad_type_off").set_active(config.getboolean('Touchpad', 'TypeOff'))
		if (config.has_option('Touchpad', 'HorizScroll')):
			self.builder.get_object("horizontal_scroll").set_active(config.getboolean('Touchpad', 'HorizScroll'))
		if (config.has_option('Touchpad', 'TapToClick')):
			self.builder.get_object("tap_to_click").set_active(config.getboolean('Touchpad', 'TapToClick'))
		
		self.builder.get_object("touchpad_disable").connect("clicked", self.touchpad_cb)
		self.builder.get_object("vertical_scroll").connect("clicked", self.touchpad_vertical)
		self.builder.get_object("horizontal_scroll").connect("clicked", self.touchpad_horizontal)
		self.builder.get_object("tap_to_click").connect("clicked", self.touchpad_tap)
		
		# off delay with type
		tp_interval = self.builder.get_object("touchpad_type_delay")
		tp_interval.set_range(100, 2000)
		tp_interval.set_increments(100, 1000)
		tp_interval.set_digits(0)
		if (config.has_option('Touchpad', 'TypeTime')):
			tp_interval.set_value(config.getint('Touchpad', 'TypeTime'))

		# accel.
		accel = self.builder.get_object("mouse_accel")
		accel.set_range(0, 10)
		accel.set_increments(1, 2)
		accel.set_digits(0)
		if (config.has_option('Mouse', 'AccFactor')):
			mouse_value = config.getint('Mouse', 'AccFactor')/10
			accel.set_value(mouse_value)
		# threshold
		threshold = self.builder.get_object("mouse_threshold")
		threshold.set_range(0, 10)
		threshold.set_increments(1, 2)
		threshold.set_digits(0)
		if (config.has_option('Mouse', 'AccThreshold')):
			mouse_threshold = config.getint('Mouse', 'AccThreshold')/10
			threshold.set_value(mouse_threshold)
		
		# left button
		
		if (config.has_option('Mouse', 'LeftHanded')):
			self.builder.get_object("left_handed").set_active(config.getboolean('Mouse', 'LeftHanded'))
		
		# connect signals of button
		self.builder.get_object("cancel").connect("clicked", Gtk.main_quit)
		self.builder.get_object("ok").connect("clicked", self.apply_cb)


	def touchpad_cb(self, w):
		pad = "synclient TouchPadOff={0}".format(int(w.get_active()))
		os.system(pad)

	def touchpad_vertical(self, w):
		vert = "synclient VertEdgeScroll={0}".format(int(w.get_active()))
		os.system(vert)

	def touchpad_horizontal(self, w):
		horiz = "synclient HorizEdgeScroll={0}".format(int(w.get_active()))
		os.system(horiz)

	def touchpad_tap(self, w):
		tap = "synclient TapButton1={0}".format(int(w.get_active()))
		os.system(tap)

	''' apply all settings '''
	def apply_cb(self, widget):
			# keyboard stuff.
			w = self.builder.get_object("kb_delay")
			delay = int(w.get_value())
			w = self.builder.get_object("kb_interval")
			interval = int(w.get_value())
			
			# impliment setting now
			print('set keyboard')
			os.system('xset r rate ' + str(delay) + ' ' + str(interval))
			
			# mouse stuff.
			w = self.builder.get_object("mouse_threshold")
			threshold = int(w.get_value())
			w = self.builder.get_object("mouse_accel")
			accel = int(w.get_value())

			# left handed?
			w = self.builder.get_object("left_handed")
			paw = str("pointer = 1 2 3")
			if(w.get_active()):
				paw = str("pointer = 3 2 1")
			# impliment setting now
			print('set mouse')
			os.system('xmodmap -e \"' + paw + '\"')
			print('set keyboard')
			os.system('xset m ' + str(accel) + ' ' + str(threshold))
			print('set touchpad')
			
			# touchpad stuff
			if(self.builder.get_object("touchpad_type_off").get_active()):
				os.system('syndaemon -kdi ' + str(self.builder.get_object("touchpad_type_delay").get_value()));

			# save the settings in LXDE session conf
			from configobj import ConfigObj
			
			ms_threshold = self.builder.get_object("mouse_threshold")
			ms_accel = self.builder.get_object("mouse_accel")
			kb_interval = self.builder.get_object("kb_interval")
			kb_delay = self.builder.get_object("kb_delay")
			type_delay = self.builder.get_object("touchpad_type_delay")
			
			
			cur_user = getpass.getuser()
			config_dir = '/home/' + cur_user + '/.config/lxsession/LXDE/'
			try:
				if not os.path.isdir(config_dir):
					os.makedirs(config_dir, 0o755)
			finally:
				config = ConfigObj(config_dir + 'desktop.conf')
			
			# set mouse config
			if ('Mouse' not in config):
				config['Mouse'] = {}
			mouse_section = config['Mouse']
			
			mouse_section['AccThreshold'] = str(int(ms_threshold.get_value() * 10))
			mouse_section['AccFactor'] = str(int(ms_accel.get_value() * 10))
			mouse_section['LeftHanded'] = str(int(self.builder.get_object("left_handed").get_active()))
			
			# set keyboard config
			if ('Keyboard' not in config):
				config['Keyboard'] = {}
			kb_section = config['Keyboard']
			kb_section['Interval'] = str(int(kb_interval.get_value()))
			kb_section['Delay'] = str(int(kb_delay.get_value()))
			
			# set touchpad config
			if ('Touchpad' not in config):
				config['Touchpad'] = {}
			tp_section = config['Touchpad']
			tp_section['Disable'] = str(int(self.builder.get_object("touchpad_disable").get_active()))
			tp_section['VertScoll'] = str(int(self.builder.get_object("vertical_scroll").get_active()))
			tp_section['TypeOff'] = str(int(self.builder.get_object("touchpad_type_off").get_active()))
			tp_section['HorizScroll'] = str(int(self.builder.get_object("horizontal_scroll").get_active()))
			tp_section['TapToClick'] = str(int(self.builder.get_object("tap_to_click").get_active()))
			tp_section['TypeTime'] = str(int(type_delay.get_value()))
			
			config.write()
			
			# delete spaces between and after =
			os.system('sed -i s/\ =\ /=/ '+ config_dir + 'desktop.conf')
			# add one empty line before [Touchpad]
			os.system('sed -i "s|\[Touchpad\]|\\n[Touchpad]|g" '+ config_dir + 'desktop.conf')
			os.system('sed -i "/^$/N;/\\n$/D" ' + config_dir + 'desktop.conf')

if __name__ == "__main__":
	mdvinput()
	Gtk.main()

